@@ -4,6 +4,7 @@ pub enum DeserializeError {
 	Custom(String),
 	Invalid(String),
 	Io(std::io::Error),
+	ParseInt(std::num::ParseIntError),
 	Unimplemented(&'static str),
 	Utf8(std::str::Utf8Error),
 }
@@ -23,6 +24,12 @@ impl From<std::io::Error> for DeserializeError {
 	}
 }
 
+impl From<std::num::ParseIntError> for DeserializeError {
+	fn from(that: std::num::ParseIntError) -> Self {
+		Self::ParseInt(that)
+	}
+}
+
 impl From<std::str::Utf8Error> for DeserializeError {
 	fn from(that: std::str::Utf8Error) -> Self {
 		Self::Utf8(that)
