@@ -1,10 +1,9 @@
 mod error; pub use error::*;
 mod key; pub(crate) use key::*;
 mod map; pub(crate) use map::*;
+mod num; pub(crate) use num::*;
 mod seq; pub(crate) use seq::*;
 
-use std::convert::TryInto as _;
-
 pub struct Deserializer<R: buffered_reader::BufferedReader<()>> {
 	input: R,
 	indent: Option<String>,
@@ -112,9 +111,39 @@ impl<R: buffered_reader::BufferedReader<()>> Deserializer<R> {
 		}
 	}
 
-	fn parse_int(&mut self) -> Result<i64, DeserializeError> {
-		Ok(self.parse_string()?.parse().unwrap())
+	fn parse_int(&mut self) -> Result<Number, DeserializeError> {
+		let s = self.parse_string()?;
+		let n = parse_num(&s)?;
+		if n.denominator != 1 {
+			return Err(DeserializeError::Invalid(format!("Expected integer, got {}", s)))
+		}
+		Ok(n)
 	}
+
+	fn parse_sint<T: std::convert::TryFrom<u128> + std::ops::Neg>(&mut self) -> Result<T, DeserializeError>
+	where
+		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
+		<T as std::ops::Neg>::Output: std::convert::Into<T>,
+	{
+		let n = self.parse_int()?;
+		let mut i = T::try_from(n.numerator).unwrap();
+		if !n.positive {
+			i = (-i).into();
+		}
+		Ok(i)
+	}
+
+	fn parse_uint<T: std::convert::TryFrom<u128>>(&mut self) -> Result<T, DeserializeError>
+	where
+		<T as std::convert::TryFrom<u128>>::Error: std::fmt::Debug,
+	{
+		let n = self.parse_int()?;
+		if !n.positive {
+			return Err(DeserializeError::Invalid(format!("Expected unsigned integer, got -{}", n.numerator)))
+		}
+		Ok(T::try_from(n.numerator).unwrap())
+	}
+
 }
 
 impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for &mut Deserializer<R> {
@@ -135,35 +164,43 @@ impl<'d, R: buffered_reader::BufferedReader<()>> serde::de::Deserializer<'d> for
 	}
 
 	fn deserialize_i8<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_i8(self.parse_sint()?)
 	}
 
 	fn deserialize_i16<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_i16(self.parse_sint()?)
 	}
 
 	fn deserialize_i32<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_i32(self.parse_sint()?)
 	}
 
 	fn deserialize_i64<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_i64(self.parse_sint()?)
+	}
+
+	fn deserialize_i128<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
+		visitor.visit_i128(self.parse_sint()?)
 	}
 
 	fn deserialize_u8<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_u8(self.parse_uint()?)
 	}
 
 	fn deserialize_u16<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_u16(self.parse_uint()?)
 	}
 
 	fn deserialize_u32<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_u32(self.parse_uint()?)
 	}
 
 	fn deserialize_u64<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
-		visitor.visit_i8(self.parse_int()?.try_into().unwrap())
+		visitor.visit_u64(self.parse_uint()?)
+	}
+
+	fn deserialize_u128<V: serde::de::Visitor<'d>>(self, visitor: V) -> Result<V::Value, DeserializeError> {
+		visitor.visit_u128(self.parse_uint()?)
 	}
 
 	fn deserialize_f32<V: serde::de::Visitor<'d>>(self, _visitor: V) -> Result<V::Value, DeserializeError> {
