@@ -1,5 +1,5 @@
-mod dict;
 mod int;
-mod list;
+mod map;
 mod option;
+mod seq;
 mod string;
